package de.intevation.flys.exports;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.util.List;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.artifactdatabase.state.ArtifactAndFacet;
import de.intevation.artifactdatabase.state.Settings;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.model.WQ;
import de.intevation.flys.collections.FLYSArtifactCollection;

import de.intevation.flys.model.Gauge;
import de.intevation.flys.model.River;
import de.intevation.flys.model.TimeInterval;
import de.intevation.flys.utils.FLYSUtils;
import de.intevation.flys.artifacts.access.RangeAccess;

public class ATExporter
implements   OutGenerator
{
    private static Logger logger = Logger.getLogger(ATExporter.class);

    public static final String DEFAULT_ENCODING = "UTF-8";

    protected WQ           data;
    protected CallContext  context;
    protected OutputStream out;
    protected FLYSArtifact master;

    protected FLYSArtifactCollection collection;


    public ATExporter() {
    }

    @Override
    public void init(Document request, OutputStream out, CallContext context) {
        this.context = context;
        this.out     = out;
    }


    @Override
    public void setMasterArtifact(Artifact master) {
        this.master = (FLYSArtifact) master;
    }

    @Override
    public void setCollection(FLYSArtifactCollection collection) {
        this.collection = collection;
    }

    @Override
    public void doOut(
        ArtifactAndFacet artifactf,
        Document attr,
        boolean  visible
    ) {
        data = (WQ)artifactf.getData(context);
    }

    @Override
    public void generate() throws IOException {

        if (data == null) {
            logger.debug("no W/Q data");
            return;
        }

        ATWriter at;
        try {
            at = new ATWriter(data);
        }
        catch (IllegalArgumentException iae) {
            logger.error("creating ATWriter failed", iae);
            throw new IOException(iae);
        }

        River river = FLYSUtils.getRiver(master);
        RangeAccess rangeAccess = new RangeAccess(master, context);
        double[] kms = rangeAccess.getLocations();

        Gauge gauge = river.determineGaugeByPosition(kms[0]);
        TimeInterval interval =
            gauge.fetchMasterDischargeTable().getTimeInterval();
        at.write(
            new OutputStreamWriter(out, DEFAULT_ENCODING),
            context.getMeta(),
            river.getName(),
            kms[0],
            gauge.getName(),
            gauge.getDatum(),
            interval.getStartTime());
    }


    /**
     * Returns an instance of <i>EmptySettings</i> currently!
     *
     * @return an instance of <i>EmptySettings</i>.
     */
    @Override
    public Settings getSettings() {
        return new EmptySettings();
    }


    /**
     * This method is not implemented!
     *
     * @param settings A settings object.
     */
    @Override
    public void setSettings(Settings settings) {
        // do nothing here
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
